module mano_sc2(clk,pc,ir,DA,AA,BA,a,b,d,MW);

	input clk;
	output [15:0] a, b, d;
	output [2:0] DA,AA,BA;
	output [15:0] pc, ir;
	output MW;
	wire [3:0] FS;
	wire [15:0] constant, data_in, data_out;
	reg [7:0] pc;
	wire [15:0] f, bx;
	wire V, C, N, Z;
	wire MB, MD, RW;
	wire PL, JB, BC;
	wire [15:0] g;
	
	// program memory
	lpm_rom	lpm_rom_component (
				.address (pc),
				.q (ir));
	defparam
		lpm_rom_component.lpm_width = 16,
		lpm_rom_component.lpm_widthad = 8,
		lpm_rom_component.lpm_address_control = "UNREGISTERED",
		lpm_rom_component.lpm_outdata = "UNREGISTERED",
		lpm_rom_component.lpm_file = "program.mif";

	// sign extend constant operand		
	assign constant[5:0] = ir[5:0];
	assign constant[15:6] = (ir[5]? 9'b111_111_111: 9'b000_000_000);
	
	// decode instruction
	assign PL = ir[15]&ir[14];
	assign FS[3:1] = ir[12:10];
	assign FS[0] = ir[9] & (~PL);
	assign JB = ir[13];
	assign DA = ir[8:6];
	// swap roles of DA and AA for branch control instructions
	assign AA = (PL? ir[8:6]: ir[5:3]);
	assign BA = ir[2:0];
	assign MB = ir[15];
	assign RW = ~ir[14];
	assign MD = ir[13];
	// we had problems with gliches creating write pulse width variations,
	// so we try to reduce the possibility MW changing state accidently by checking six bits
	assign MW = (ir[15:10]==(6'b010_000)? 1'b1: 1'b0);
	
	assign BC = (ir[9]? N: Z);
	
	always @(posedge clk)
	begin
		if (PL & JB) 
		begin 
			if (ir[9]) pc <= a[7:0];
			else pc <= ir[7:0];
		end
		else if (PL & BC) pc <= pc + constant[7:0];
		else pc <= pc + 1'b1;
	end
	

	// register file
	reg_file U2(clk, RW, AA, BA, DA, a, b, d);
	assign data_out = (MB? constant: b);
	
	
	// data memory
	lpm_ram_dq	lpm_ram_dq_component (
			.address (a[7:0]),
			.inclock (clk),
			.data (data_out),
			.we (MW),
			.q (data_in));
	defparam
		lpm_ram_dq_component.lpm_width = 16,
		lpm_ram_dq_component.lpm_widthad = 8,
		lpm_ram_dq_component.lpm_indata = "REGISTERED",
		lpm_ram_dq_component.lpm_address_control = "UNREGISTERED",
		lpm_ram_dq_component.lpm_outdata = "UNREGISTERED",
		lpm_ram_dq_component.lpm_file = "data.mif",
		lpm_ram_dq_component.lpm_hint = "USE_EAB=ON";

	// function unit
	func U3(a,data_out,FS,f,V,C,N,Z);
	assign d = (MD? data_in: f);

endmodule

